"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getActivateState = getActivateState;
exports["default"] = exports.mergedActivitiesFrom = exports.localIdentifierFrom = exports.dateCreatedAndModifiedsByEntityIdentifierFrom = exports.entitiesByActivityIdentifierFrom = void 0;

var _getNormalizedMergedState = _interopRequireDefault(require("./getNormalizedMergedState"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var entitiesByActivityIdentifierFrom = function entitiesByActivityIdentifierFrom(state, activities) {
  var entitiesByActivityIdentifier = {};
  activities.forEach(function (activity) {
    var stateKey = activity.stateKey;
    var entity = (state[stateKey] || []).find(function (e) {
      return e.activityIdentifier === activity.entityIdentifier;
    });

    if (entity) {
      entitiesByActivityIdentifier[activity.entityIdentifier] = entity;
    }
  });
  return entitiesByActivityIdentifier;
};

exports.entitiesByActivityIdentifierFrom = entitiesByActivityIdentifierFrom;

var dateCreatedAndModifiedsByEntityIdentifierFrom = function dateCreatedAndModifiedsByEntityIdentifierFrom(state, activities, entitiesByActivityIdentifier) {
  var entityDateCreatedsByIdentifier = {};
  var entityDateModifiedsByIdentifier = {};
  activities.forEach(function (activity) {
    var entity = entitiesByActivityIdentifier[activity.entityIdentifier];

    if (typeof entityDateCreatedsByIdentifier[activity.entityIdentifier] === 'undefined') {
      if (entity) {
        entityDateCreatedsByIdentifier[activity.entityIdentifier] = entity.dateCreated;
      } else {
        entityDateCreatedsByIdentifier[activity.entityIdentifier] = activity.dateCreated;
        return;
      }
    }

    if (entityDateCreatedsByIdentifier[activity.entityIdentifier] !== activity.dateCreated) {
      if (!entity || !entity.dateModified || new Date(activity.dateCreated) >= new Date(entity.dateModified)) {
        entityDateModifiedsByIdentifier[activity.entityIdentifier] = activity.dateCreated;
      } else {
        entityDateModifiedsByIdentifier[activity.entityIdentifier] = entity.dateModified;
      }
    }
  });
  return {
    entityDateCreatedsByIdentifier: entityDateCreatedsByIdentifier,
    entityDateModifiedsByIdentifier: entityDateModifiedsByIdentifier
  };
};

exports.dateCreatedAndModifiedsByEntityIdentifierFrom = dateCreatedAndModifiedsByEntityIdentifierFrom;

var localIdentifierFrom = function localIdentifierFrom(activity) {
  return "".concat(activity.entityIdentifier, "/").concat(activity.dateCreated);
};

exports.localIdentifierFrom = localIdentifierFrom;

var mergedActivitiesFrom = function mergedActivitiesFrom(stateActivities, patchActivities) {
  return (0, _getNormalizedMergedState["default"])({
    __activities__: stateActivities
  }, {
    __activities__: patchActivities
  }, {
    getDatumIdKey: function getDatumIdKey() {
      return 'localIdentifier';
    },
    getDatumIdValue: function getDatumIdValue(activity) {
      return activity.id || localIdentifierFrom(activity);
    },
    isMergingDatum: true
  }).__activities__;
};

exports.mergedActivitiesFrom = mergedActivitiesFrom;

var activatedEntityFrom = function activatedEntityFrom(activity) {
  var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var entityDateCreatedsByIdentifier = config.entityDateCreatedsByIdentifier,
      entityDateModifiedsByIdentifier = config.entityDateModifiedsByIdentifier;

  var activatedEntity = _objectSpread({
    activityIdentifier: activity.entityIdentifier
  }, activity.patch);

  if (entityDateCreatedsByIdentifier) {
    activatedEntity.dateCreated = entityDateCreatedsByIdentifier[activity.entityIdentifier];
  }

  if (entityDateModifiedsByIdentifier) {
    activatedEntity.dateModified = entityDateModifiedsByIdentifier[activity.entityIdentifier] || null;
  }

  return activatedEntity;
};

function getActivateState(state, activities) {
  var config = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  return activities.reduce(function (aggregation, activity) {
    return _objectSpread(_objectSpread({}, aggregation), (0, _getNormalizedMergedState["default"])(aggregation, _defineProperty({}, activity.stateKey, [activatedEntityFrom(activity, config)]), {
      getDatumIdKey: function getDatumIdKey() {
        return 'activityIdentifier';
      },
      getDatumIdValue: function getDatumIdValue(entity) {
        return entity.activityIdentifier;
      },
      isMergingDatum: true
    }));
  }, state);
}

var _default = getActivateState;
exports["default"] = _default;