"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getMergedData = getMergedData;
exports["default"] = exports.merge = void 0;

var _lodash = _interopRequireDefault(require("lodash.uniq"));

var _lodash2 = _interopRequireDefault(require("lodash.uniqby"));

var _getUnifiedData = require("./getUnifiedData");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var merge = function merge(target, source) {
  var _loop = function _loop() {
    var key = _Object$keys[_i];

    if (source[key] instanceof Object) {
      if (Array.isArray(source[key])) {
        if (source[key][0]) {
          if (!(source[key][0] instanceof Object) || Array.isArray(source[key][0])) {
            target[key] = source[key];
            return "continue";
          }
        }

        target[key] = source[key].map(function (s, index) {
          return merge(_objectSpread({}, target[key] && target[key][index]), s);
        });
      } else {
        target[key] = merge(_objectSpread({}, target[key]), source[key]);
      }
    } else {
      target[key] = source[key];
    }
  };

  for (var _i = 0, _Object$keys = Object.keys(source); _i < _Object$keys.length; _i++) {
    var _ret = _loop();

    if (_ret === "continue") continue;
  }

  return target;
};

exports.merge = merge;

function getMergedData(nextData, previousData, config) {
  if (!previousData) {
    return nextData;
  }

  var cacheKey = config.cacheKey,
      isMergingDatum = config.isMergingDatum,
      isMutatingDatum = config.isMutatingDatum,
      resolve = config.resolve;
  var getDatumIdValue = config.getDatumIdValue || _getUnifiedData.getDefaultDatumIdValue;
  var isMutatingArray = typeof config.isMutatingArray === 'undefined' ? true : config.isMutatingArray;
  var mergedData = isMutatingArray ? _toConsumableArray(previousData) : previousData; // for each datum we are going to assign (by merging or not) them into
  // their right place in the resolved array

  nextData.forEach(function (nextDatum) {
    var previousIndex = previousData.findIndex(function (previousDatum) {
      if (getDatumIdValue(previousDatum) === getDatumIdValue(nextDatum)) return true;

      if (typeof previousDatum.activityIdentifier !== 'undefined' && previousDatum.activityIdentifier !== null && typeof nextDatum.activityIdentifier !== 'undefined' && nextDatum.activityIdentifier !== null) {
        return previousDatum.activityIdentifier === nextDatum.activityIdentifier;
      }

      return false;
    });
    var previousDatum = previousData[previousIndex];
    var resolvedIndex = previousIndex === -1 ? mergedData.length : previousIndex;
    var datum;

    if (resolve) {
      datum = resolve(nextDatum, previousDatum, {
        previousData: previousData,
        nextData: nextData,
        config: config
      });
    } else if (isMutatingDatum) {
      datum = Object.assign({}, isMergingDatum && previousDatum, nextDatum);
    } else if (isMergingDatum) {
      datum = previousIndex !== -1 ? merge(previousDatum, nextDatum) : nextDatum;
    } else {
      datum = nextDatum;
    }

    if (previousDatum && previousDatum.__normalizers__ && nextDatum.__normalizers__) {
      datum.__normalizers__ = (0, _lodash2["default"])(previousDatum.__normalizers__.concat(nextDatum.__normalizers__), function (normalizer) {
        return "".concat(normalizer.datumKey || '', "-").concat(normalizer.stateKey || '');
      });
    }

    if (previousDatum && previousDatum.__tags__ && nextDatum.__tags__) {
      datum.__tags__ = (0, _lodash["default"])(previousDatum.__tags__.concat(nextDatum.__tags__));
    }

    mergedData[resolvedIndex] = datum;

    if (cacheKey) {
      var previousCache = (previousDatum || {})[cacheKey] || {};

      var cache = _objectSpread(_objectSpread({}, previousCache), nextDatum);

      if (cache[cacheKey]) {
        delete cache[cacheKey];
      }

      if (cache.__normalizers__) {
        delete cache.__normalizers__;
      }

      if (cache.__tags__) {
        delete cache.__tags__;
      }

      mergedData[resolvedIndex][cacheKey] = cache;
    }
  });
  return mergedData;
}

var _default = getMergedData;
exports["default"] = _default;