"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.hydratedActivityFrom = hydratedActivityFrom;
exports.getNormalizedActivatedState = getNormalizedActivatedState;
exports["default"] = exports.stateWithoutDeletedEntitiesFrom = exports.deletedActivityIdentifiersByStateKeyFrom = exports.notDeletedActivitiesFrom = exports.sortedHydratedActivitiesFrom = exports.forceActivitiesWithStrictIncreasingDateCreated = void 0;

var _pluralize = _interopRequireDefault(require("pluralize"));

var _getActivateState = _interopRequireWildcard(require("./getActivateState"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var stateKeyFrom = function stateKeyFrom(activity) {
  var localStateKey = activity.localStateKey;

  if (!localStateKey) {
    if (activity.tableName) {
      localStateKey = (0, _pluralize["default"])(activity.tableName, 2);
    } else if (activity.modelName) {
      localStateKey = (0, _pluralize["default"])("".concat(activity.modelName[0].toLowerCase()).concat(activity.modelName.slice(1)), 2);
    } else {
      console.warn('Missing localStateKey or tableName or modelName for that activity.');
    }
  }

  return localStateKey;
};

function hydratedActivityFrom(activity) {
  return _objectSpread(_objectSpread({}, activity), {}, {
    dateCreated: activity.dateCreated || new Date().toISOString(),
    deprecation: null,
    bothLocalAndRemoteEntityHaveBeenModified: false,
    patch: _objectSpread({}, activity.patch),
    stateKey: stateKeyFrom(activity)
  });
}

var forceActivitiesWithStrictIncreasingDateCreated = function forceActivitiesWithStrictIncreasingDateCreated(activities) {
  var lastDateCreatedByIdentifier = {};
  activities.forEach(function (activity) {
    var lastDateCreated = lastDateCreatedByIdentifier[activity.entityIdentifier];

    if (lastDateCreated >= activity.dateCreated) {
      var dateTimePlusOneMillisecond = new Date(lastDateCreated).getTime() + 1;
      activity.dateCreated = new Date(dateTimePlusOneMillisecond).toISOString();
    }

    lastDateCreatedByIdentifier[activity.entityIdentifier] = activity.dateCreated;
  });
};

exports.forceActivitiesWithStrictIncreasingDateCreated = forceActivitiesWithStrictIncreasingDateCreated;

var sortedHydratedActivitiesFrom = function sortedHydratedActivitiesFrom(activities) {
  var hydratedSortedActivities = (activities || []).map(hydratedActivityFrom);
  hydratedSortedActivities.sort(function (activity1, activity2) {
    return new Date(activity1.dateCreated) < new Date(activity2.dateCreated) ? -1 : 1;
  });
  forceActivitiesWithStrictIncreasingDateCreated(hydratedSortedActivities);
  return hydratedSortedActivities;
};

exports.sortedHydratedActivitiesFrom = sortedHydratedActivitiesFrom;

var notDeletedActivitiesFrom = function notDeletedActivitiesFrom(activities, deletedActivityIdentifiersByStateKey) {
  var deletedActivityIdentifiers = Object.values(deletedActivityIdentifiersByStateKey).reduce(function (agg, list) {
    return agg.concat(list);
  }, []);
  return activities.filter(function (activity) {
    return !deletedActivityIdentifiers.includes(activity.entityIdentifier);
  });
};

exports.notDeletedActivitiesFrom = notDeletedActivitiesFrom;

var deletedActivityIdentifiersByStateKeyFrom = function deletedActivityIdentifiersByStateKeyFrom(state, activities) {
  var deletedActivityIdentifiersByStateKey = {};
  activities.filter(function (activity) {
    return activity.verb === 'delete';
  }).forEach(function (activity) {
    var activityIdentifier = activity.entityIdentifier;
    var stateKey = activity.stateKey;

    if (!deletedActivityIdentifiersByStateKey[stateKey]) {
      deletedActivityIdentifiersByStateKey[stateKey] = [activityIdentifier];
    } else {
      deletedActivityIdentifiersByStateKey[stateKey].push(activityIdentifier);
    }
  });
  return deletedActivityIdentifiersByStateKey;
};

exports.deletedActivityIdentifiersByStateKeyFrom = deletedActivityIdentifiersByStateKeyFrom;

var stateWithoutDeletedEntitiesFrom = function stateWithoutDeletedEntitiesFrom(state, deletedActivityIdentifiersByStateKey) {
  var stateWithoutDeletedEntities = _objectSpread({}, state);

  Object.keys(deletedActivityIdentifiersByStateKey).forEach(function (localStateKey) {
    if (!stateWithoutDeletedEntities[localStateKey]) {
      return;
    }

    stateWithoutDeletedEntities[localStateKey] = stateWithoutDeletedEntities[localStateKey].filter(function (entity) {
      return !deletedActivityIdentifiersByStateKey[localStateKey].includes(entity.activityIdentifier);
    });

    if (!stateWithoutDeletedEntities[localStateKey].length) {
      delete stateWithoutDeletedEntities[localStateKey];
    }
  });
  return stateWithoutDeletedEntities;
};

exports.stateWithoutDeletedEntitiesFrom = stateWithoutDeletedEntitiesFrom;

function getNormalizedActivatedState(state, activities) {
  var sortedHydratedActivities = sortedHydratedActivitiesFrom(activities);
  var deletedActivityIdentifiersByStateKey = deletedActivityIdentifiersByStateKeyFrom(state, sortedHydratedActivities);
  var notDeletedActivities = notDeletedActivitiesFrom(sortedHydratedActivities, deletedActivityIdentifiersByStateKey);
  var stateWithoutDeletedEntities = stateWithoutDeletedEntitiesFrom(state, deletedActivityIdentifiersByStateKey);
  var entitiesByActivityIdentifier = (0, _getActivateState.entitiesByActivityIdentifierFrom)(state, sortedHydratedActivities);

  var _dateCreatedAndModifi = (0, _getActivateState.dateCreatedAndModifiedsByEntityIdentifierFrom)(state, notDeletedActivities, entitiesByActivityIdentifier),
      entityDateCreatedsByIdentifier = _dateCreatedAndModifi.entityDateCreatedsByIdentifier,
      entityDateModifiedsByIdentifier = _dateCreatedAndModifi.entityDateModifiedsByIdentifier;

  var activateState = (0, _getActivateState["default"])(stateWithoutDeletedEntities, notDeletedActivities, {
    entityDateCreatedsByIdentifier: entityDateCreatedsByIdentifier,
    entityDateModifiedsByIdentifier: entityDateModifiedsByIdentifier
  });
  activateState.__activities__ = (0, _getActivateState.mergedActivitiesFrom)(state.__activities__, sortedHydratedActivities);
  return activateState;
}

var _default = getNormalizedActivatedState;
exports["default"] = _default;