"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = exports.createDataReducer = void 0;

var _getNormalizedActivatedState = _interopRequireDefault(require("../../normalize/getNormalizedActivatedState"));

var _getNormalizedDeletedState = _interopRequireDefault(require("../../normalize/getNormalizedDeletedState"));

var _getNormalizedMergedState = _interopRequireDefault(require("../../normalize/getNormalizedMergedState"));

var _getNormalizedSucceedState = _interopRequireDefault(require("../../normalize/getNormalizedSucceedState"));

var _actions = require("./actions");

var _getDeletedPatchByActivityTags = _interopRequireDefault(require("./getDeletedPatchByActivityTags"));

var _reinitializeState = _interopRequireDefault(require("./reinitializeState"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var createDataReducer = function createDataReducer() {
  var initialState = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var reducer = function reducer() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
    var action = arguments.length > 1 ? arguments[1] : undefined;

    if (action.type === _actions.ACTIVATE_DATA) {
      return (0, _getNormalizedActivatedState["default"])(state, action.activities);
    }

    if (action.type === _actions.ASSIGN_DATA) {
      return Object.keys(action.patch || {}).length ? _objectSpread(_objectSpread({}, state), action.patch) : state;
    }

    if (action.type === _actions.DELETE_DATA) {
      var patch = action.patch || state;

      if (action.config.tags) {
        patch = (0, _getDeletedPatchByActivityTags["default"])(patch, action.config.tags);
      }

      var deleteState = (0, _getNormalizedDeletedState["default"])(state, patch, action.config);
      return Object.keys(deleteState).length ? _objectSpread(_objectSpread({}, state), deleteState) : state;
    }

    if (action.type === _actions.MERGE_DATA) {
      var mergeState = (0, _getNormalizedMergedState["default"])(state, action.patch, action.config);
      return Object.keys(mergeState).length ? _objectSpread(_objectSpread({}, state), mergeState) : state;
    }

    if (action.type === _actions.REINITIALIZE_DATA) {
      return (0, _reinitializeState["default"])(state, initialState, action.config);
    }

    if (/SUCCESS_DATA_(DELETE|GET|POST|PUT|PATCH)_(.*)/.test(action.type)) {
      return (0, _getNormalizedSucceedState["default"])(state, action);
    }

    return state;
  };

  return reducer;
};

exports.createDataReducer = createDataReducer;
var _default = createDataReducer;
exports["default"] = _default;